-- ============================================================================
-- == HTGUI - Provides support to HalonaTools for managing GUI resources.    ==
-- ============================================================================

HTGUI        = LibStub("AceAddon-3.0"):NewAddon("HTGUI", "AceConsole-3.0", "AceEvent-3.0")
local L      = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local AceGUI = LibStub("AceGUI-3.0")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTGUI:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTGUI:OnEnable()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTGUI:OnDisable()
end

-- ============================================================================
-- == AboutHalonaTools()                                                     ==
-- ==                                                                        ==
-- == This function displays a standard "About" window, providing            ==
-- == information about the HalonaTools addon.                               ==
-- ============================================================================

function HTGUI:About()

  local frame = AceGUI:Create("Frame")
  frame:SetTitle("About")
  frame:SetStatusText("(c) aLL I EVER WANTED productions, 2011.")
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)

end

-- ============================================================================
-- == ShowChange(fieldName, value)                                           ==
-- ==                                                                        ==
-- == This function displays a message indicating that a configuration value ==
-- == has changed, indicating which field was changed and what the new value ==
-- == of the field has become.                                               ==
-- ============================================================================

function HTGUI:ShowChange(fieldName, value)

  HTConsole:SendChatMessage("[HT] - " .. fieldName .. " - " .. value, "CHAT", nil, nil)

end

-- ============================================================================
-- == BuildLengthAdvisoryLabel(name, value, maxLength)                       ==
-- ==                                                                        ==
-- == This function builds a text string that may be used as a field label   ==
-- == that incorporates advice on the current field length and the maximum   ==
-- == permitted field length.                                                ==
-- ==                                                                        ==
-- == Example: "RMS Message (32/255)"                                        ==
-- ============================================================================

function HTGUI:BuildLengthAdvisoryLabel(name, value, maxLength)

  local valueLength = string.len(tostring(value))
  local labelText = name .. " (" .. tostring(valueLength) .. "/" .. tostring(maxLength) .. ")"

  return labelText

end

-- ============================================================================
-- == Config()                                                               ==
-- ==                                                                        ==
-- == This function displays the configuration window for the HalonaTools    ==
-- == addon.                                                                 ==
-- ============================================================================

function HTGUI:Config()

  -- ==========================================================================
  -- == DrawGroup1 - Local Function - Draws tab-group #1. (Off-Time GMOTD)   ==
  -- ==========================================================================
  local function DrawGroup1(container)
    container:SetFullHeight(true)

    local desc = AceGUI:Create("Label")
    desc:SetText("Off-Time GMOTD")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Show Current Off-Time GMOTD")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTGUI:ShowChange("Off-Time GMOTD", HTConfig:GetDefaultOfftimeGMOTD(nil)) end)
    container:AddChild(button)

    local button = AceGUI:Create("Button")
    button:SetText("Reset to Default")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConfig:SetDefaultOfftimeGMOTD(L["ht-raid-session-end-gmotd-01"], L["ht-raid-session-end-gmotd-01"]) container:ReleaseChildren() DrawGroup1(container) end)
    container:AddChild(button)

    local editbox = AceGUI:Create("MultiLineEditBox")
    editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Off-Time GMOTD", HTConfig:GetDefaultOfftimeGMOTD(nil), 255))
    editbox:SetWidth(600)
    editbox:SetNumLines(4)
    editbox:SetText(HTConfig:GetDefaultOfftimeGMOTD(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetDefaultOfftimeGMOTD(text, text) end)
    editbox:SetCallback("OnTextChanged", function(widget, event, text) editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Off-Time GMOTD", HTConfig:GetDefaultOfftimeGMOTD(nil), 255)) container:DoLayout() end)
    container:AddChild(editbox)
  end

  -- ==========================================================================
  -- == DrawGroup2 - Local Function - Draws tab-group #2. (Recruitment)      ==
  -- ==========================================================================
  local function DrawGroup2(container)
    container:SetFullHeight(true)

    local desc = AceGUI:Create("Label")
    desc:SetText("Recruitment Message Sender")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Show Current RMS Message")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTGUI:ShowChange("RMS Message", HTConfig:GetRecruitmentMessage(nil)) end)
    container:AddChild(button)

    local button = AceGUI:Create("Button")
    button:SetText("Reset to Default")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConfig:SetRecruitmentMessage(L["cfg-default-recruitment-msg"], L["cfg-default-recruitment-msg"]) container:ReleaseChildren() DrawGroup2(container) end)
    container:AddChild(button)

    local editbox = AceGUI:Create("MultiLineEditBox")
    editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Recruitment Message", HTConfig:GetRecruitmentMessage(nil), 255))
    editbox:SetWidth(600)
    editbox:SetNumLines(4)
    editbox:SetText(HTConfig:GetRecruitmentMessage(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRecruitmentMessage(text, text) end)
    editbox:SetCallback("OnTextChanged", function(widget, event, text) editbox:SetLabel(HTGUI:BuildLengthAdvisoryLabel("Recruitment Message", HTConfig:GetRecruitmentMessage(nil), 255)) container:DoLayout() end)
    container:AddChild(editbox)
  end

  -- ==========================================================================
  -- == DrawGroup3 - Local Function - Draws tab-group #3. (General Config.)  ==
  -- ==========================================================================
  local function DrawGroup3(container)
    container:SetFullHeight(true)

    local desc = AceGUI:Create("Label")
    desc:SetText("Console")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local conTextMaxWidthSlider = AceGUI:Create("Slider")
    conTextMaxWidthSlider:SetValue(HTConfig:GetChatWindowTextMaxWidth(nil))
    conTextMaxWidthSlider:SetSliderValues(30, 100, 5)
    conTextMaxWidthSlider:SetLabel("Max Chat-Window Text Width")
    conTextMaxWidthSlider:SetCallback("OnMouseUp", function(widget, event, text) HTConfig:SetChatWindowTextMaxWidth(tonumber(text), tonumber(text)) end)
    container:AddChild(conTextMaxWidthSlider)

    local button = AceGUI:Create("Button")
    button:SetText("Test Setting")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConsole:SendTitleBar("CHAT") end)
    container:AddChild(button)
  end

  -- ==========================================================================
  -- == DrawGroup4 - Local Function - Draws tab-group #4. (RankTags)         ==
  -- ==========================================================================
  local function DrawGroup4(container)
    container:SetFullHeight(true)

    local desc = AceGUI:Create("Label")
    desc:SetText("RankTags")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Reset Original Settings")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConfig:RefreshHalonaRankTags() HTConfig:ShowRankTags() container:ReleaseChildren() DrawGroup4(container) end)
    container:AddChild(button)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 01: "..HTCommon:GetGuildRankNameByIndex(1))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags01(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags01(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 02: "..HTCommon:GetGuildRankNameByIndex(2))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags02(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags02(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 03: "..HTCommon:GetGuildRankNameByIndex(3))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags03(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags03(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 04: "..HTCommon:GetGuildRankNameByIndex(4))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags04(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags04(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 05: "..HTCommon:GetGuildRankNameByIndex(5))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags05(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags05(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 06: "..HTCommon:GetGuildRankNameByIndex(6))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags06(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags06(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 07: "..HTCommon:GetGuildRankNameByIndex(7))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags07(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags07(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 08: "..HTCommon:GetGuildRankNameByIndex(8))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags08(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags08(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 09: "..HTCommon:GetGuildRankNameByIndex(9))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags09(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags09(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Rank 10: "..HTCommon:GetGuildRankNameByIndex(10))
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetRankTags10(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetRankTags10(text, text) end)
    container:AddChild(editbox)
  end

  -- ==========================================================================
  -- == DrawGroup5 - Local Function - Draws tab-group #5. (VOIP Details) ======
  -- ==========================================================================
  local function DrawGroup5(container)
    container:SetFullHeight(true)

    local desc = AceGUI:Create("Label")
    desc:SetText("VOIP-Account-Details")
    desc:SetFullWidth(true)
    container:AddChild(desc)

    local button = AceGUI:Create("Button")
    button:SetText("Set Defaults for Halona")
    button:SetWidth(200)
    button:SetCallback("OnClick", function() HTConfig:SetVoipDetailsForHalonaMembers() container:ReleaseChildren() DrawGroup5(container) end)
    container:AddChild(button)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Type")
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetVoipType(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetVoipType(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Hostname")
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetVoipHostname(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetVoipHostname(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Port Number")
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetVoipPortNumber(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetVoipPortNumber(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Username")
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetVoipUsername(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetVoipUsername(text, text) end)
    container:AddChild(editbox)

    local editbox = AceGUI:Create("EditBox")
    editbox:SetLabel("Password")
    editbox:SetWidth(600)
    editbox:SetText(HTConfig:GetVoipPassword(nil))
    editbox:SetMaxLetters(255)
    editbox:SetCallback("OnEnterPressed", function(widget, event, text) HTConfig:SetVoipPassword(text, text) end)
    container:AddChild(editbox)

  end

  -- ==========================================================================
  -- == SelectGroup - Local Function - Draws a tab when selected.            ==
  -- ==========================================================================
  local function SelectGroup(container, event, group)
     container:ReleaseChildren()
     if group == "tab1" then
        DrawGroup1(container)
     elseif group == "tab2" then
        DrawGroup2(container)
     elseif group == "tab3" then
        DrawGroup3(container)
     elseif group == "tab4" then
        DrawGroup4(container)
     elseif group == "tab5" then
        DrawGroup5(container)
     end
  end

  -- ==========================================================================
  -- == Main functionality - Create the frames that will host the tab-group. ==
  -- ==========================================================================
  local frame = AceGUI:Create("Frame")
  frame:SetHeight(700)
  frame:SetTitle("HT-Config")
  frame:SetStatusText("HalonaTools Configuration GUI")
  frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
  -- Fill Layout - the TabGroup widget will fill the whole frame
  frame:SetLayout("Fill")

  scrollcontainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
  scrollcontainer:SetHeight(650)
  scrollcontainer:SetLayout("Fill") -- important!

  frame:AddChild(scrollcontainer)

  -- Add a scrolling frame inside the main frame.
  local scrollFrame = AceGUI:Create("ScrollFrame")
  scrollFrame:SetHeight(650)
  scrollFrame:SetLayout("Fill")
  scrollcontainer:AddChild(scrollFrame)

  -- frame:AddChild(scrollFrame)

  -- Create the TabGroup
  local tab =  AceGUI:Create("TabGroup")
  tab:SetHeight(650)
  tab:SetLayout("Flow")
  tab:SetFullHeight(true)

  -- Setup which tabs to show
  tab:SetTabs({{text="Off-Time GMOTD", value="tab1"}, {text="Recruitment Message", value="tab2"}, {text="Console Config.", value="tab3"}, {text="RankTags", value="tab4"}, {text="VOIP Details", value="tab5"}})
  -- Register callback
  tab:SetCallback("OnGroupSelected", SelectGroup)
  -- Set initial Tab (this will fire the OnGroupSelected callback)
  tab:SelectTab("tab1")

  -- add to the frame container
  scrollFrame:AddChild(tab)

end


-- ============================================================================
-- == PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)           ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid invitation from the current raid inviter and    ==
-- == processes the response they make.                                      ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidInvitation(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-ria-window-msg-01"], "RIA_INVITER", raidInviter)
  local joinMessage   = L["gui-ria-msg-first"] .. L["gui-ria-join-msg-01"]
  local cancelMessage = L["gui-ria-msg-first"] .. L["gui-ria-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RIA_KEYWORD", L["gui-ria-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RIA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-ria-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-ria-window-btn-yes"],
    button2      = L["gui-ria-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-ria-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-ria-window-title"])
    PlaySound("DwarfExploration")
  end

end

-- ============================================================================
-- == PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)         ==
-- ==                                                                        ==
-- == This function displays a static popup dialog asking the player if they ==
-- == wish to request a raid queue listing from the current raid queue       ==
-- == manager and processes the response they make.                          ==
-- ============================================================================

function HTGUI:PopupAskRequestRaidQueueListing(raidInviter, messageAnnouncer)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the text strings...                                    --
  -- == ----------------------------------------------------------------------
  local windowTitle   = string.gsub(L["gui-rqa-window-msg-01"], "RQA_INVITER", raidInviter)
  local joinMessage   = L["gui-rqa-msg-first"] .. L["gui-rqa-join-msg-01"]
  local cancelMessage = L["gui-rqa-msg-first"] .. L["gui-rqa-cancel-msg-01"]

  cancelMessage = string.gsub(cancelMessage, "RQA_KEYWORD", L["gui-rqa-keyword"])
  cancelMessage = string.gsub(cancelMessage, "RQA_INVITER", raidInviter)

  -- == ----------------------------------------------------------------------
  -- == -- Configure the popup window...                                    --
  -- == ----------------------------------------------------------------------
  StaticPopupDialogs[L["gui-rqa-window-title"]] = {

    text         = windowTitle,
    button1      = L["gui-rqa-window-btn-yes"],
    button2      = L["gui-rqa-window-btn-no"],
    timeout      = 30,
    whileDead    = 1,
    hideOnEscape = 0,

    OnAccept = function()
      HTConsole:SendChatMessage(L["gui-rqa-keyword"], "WHISPER", nil, raidInviter)
      HTConsole:SendChatMessage(joinMessage, "CHAT", nil, nil)
    end,

    OnCancel = function()
      HTConsole:SendChatMessage(cancelMessage, "CHAT", nil, nil)
    end
  }

  -- == ----------------------------------------------------------------------
  -- == -- Display the popup window...                                      --
  -- == ----------------------------------------------------------------------
  if HTConfig:GetIsEnabledAutoRIAResponse(nil) then
    StaticPopup_Show (L["gui-rqa-window-title"])
    PlaySound("DwarfExploration")
  end

end
